@@ -26,10 +26,10 @@
  *
  * HTTP client-side authentication handler.
  *
- * [class@AuthManager] is the [iface@SessionFeature] that handles HTTP
+ * #SoupAuthManager is the [iface@SessionFeature] that handles HTTP
  * authentication for a [class@Session].
  *
- * A [class@AuthManager] is added to the session by default, and normally
+ * A #SoupAuthManager is added to the session by default, and normally
  * you don't need to worry about it at all. However, if you want to
  * disable HTTP authentication, you can remove the feature from the
  * session with [method@Session.remove_feature_by_type] or disable it on
@@ -40,7 +40,7 @@
  *
  * (Although this type has only been publicly visible since libsoup 2.42, it has
  * always existed in the background, and you can use `g_type_from_name
- * ("SoupAuthManager")` to get its [alias@GObject.Type] in earlier releases.)
+ * ("SoupAuthManager")` to get its [alias@GLib.Type] in earlier releases.)
  **/
 static void soup_auth_manager_session_feature_init (SoupSessionFeatureInterface *feature_interface, gpointer interface_data);
 
@@ -441,7 +441,7 @@ update_authorization_header (SoupMessage *msg, SoupAuth *auth, gboolean is_proxy
 	if (!token)
 		return;
 
-	soup_message_headers_replace_common (soup_message_get_request_headers (msg), authorization_header, token, SOUP_HEADER_VALUE_TRUSTED);
+	soup_message_headers_replace_common (soup_message_get_request_headers (msg), authorization_header, token);
 	g_free (token);
 }
 
