@@ -22,15 +22,15 @@
  *
  * An HTTP server request and response pair.
  *
- * A [class@ServerMessage] represents an HTTP message that is being sent or
+ * A SoupServerMessage represents an HTTP message that is being sent or
  * received on a [class@Server].
  *
- * [class@Server] will create [class@ServerMessage]s automatically for
+ * [class@Server] will create `SoupServerMessage`s automatically for
  * incoming requests, which your application will receive via handlers.
  *
  * Note that libsoup's terminology here does not quite match the HTTP
  * specification: in RFC 2616, an "HTTP-message" is *either* a Request, *or* a
- * Response. In libsoup, a [class@ServerMessage] combines both the request and the
+ * Response. In libsoup, a #SoupServerMessage combines both the request and the
  * response.
  **/
 
@@ -393,7 +393,7 @@ soup_server_message_class_init (SoupServerMessageClass *klass)
 	/**
 	 * SoupServerMessage:tls-peer-certificate-errors:
 	 *
-	 * The verification errors on [property@ServerMessage:tls-peer-certificate]
+	 * The verification errors on #SoupServerMessage:tls-peer-certificate
 	 *
 	 * Since: 3.2
 	 */
@@ -921,7 +921,7 @@ soup_server_message_get_uri (SoupServerMessage *msg)
  *   a data buffer containing the body of the message response.
  * @resp_length: the byte length of @resp_body.
  *
- * Convenience function to set the response body of a [class@ServerMessage]. If
+ * Convenience function to set the response body of a #SoupServerMessage. If
  * @content_type is %NULL, the response body must be empty as well.
  */
 void
@@ -939,7 +939,7 @@ soup_server_message_set_response (SoupServerMessage *msg,
 
                 soup_message_headers_replace_common (msg->response_headers,
                                                      SOUP_HEADER_CONTENT_TYPE,
-                                                     content_type, SOUP_HEADER_VALUE_UNTRUSTED);
+                                                     content_type);
                 soup_message_body_append (msg->response_body, resp_use,
                                           resp_body, resp_length);
         } else {
@@ -980,7 +980,7 @@ soup_server_message_set_redirect (SoupServerMessage *msg,
 	soup_server_message_set_status (msg, status_code, NULL);
 	location_str = g_uri_to_string (location);
 	soup_message_headers_replace_common (msg->response_headers, SOUP_HEADER_LOCATION,
-                                             location_str, SOUP_HEADER_VALUE_UNTRUSTED);
+                                             location_str);
 	g_free (location_str);
 	g_uri_unref (location);
 }
@@ -1018,7 +1018,7 @@ soup_server_message_get_socket (SoupServerMessage *msg)
  *
  * Returns: (nullable) (transfer none): the #GSocketAddress
  *   associated with the remote end of a connection, it may be
- *   %NULL if you used [method@Server.accept_iostream].
+ *   %NULL if you used [class@Server.accept_iostream].
  */
 GSocketAddress *
 soup_server_message_get_remote_address (SoupServerMessage *msg)
@@ -1081,7 +1081,7 @@ soup_server_message_get_remote_host (SoupServerMessage *msg)
  * soup_server_message_steal_connection:
  * @msg: a #SoupServerMessage
  *
- * "Steals" the HTTP connection associated with @msg from its [class@Server]. This
+ * "Steals" the HTTP connection associated with @msg from its #SoupServer. This
  * happens immediately, regardless of the current state of the connection; if
  * the response to @msg has not yet finished being sent, then it will be
  * discarded; you can steal the connection from a
