@@ -27,12 +27,12 @@
  *
  * Debug logging support
  *
- * [class@Logger] watches a [class@Session] and logs the HTTP traffic that
+ * #SoupLogger watches a [class@Session] and logs the HTTP traffic that
  * it generates, for debugging purposes. Many applications use an
  * environment variable to determine whether or not to use
- * [class@Logger], and to determine the amount of debugging output.
+ * #SoupLogger, and to determine the amount of debugging output.
  *
- * To use [class@Logger], first create a logger with [ctor@Logger.new], optionally
+ * To use #SoupLogger, first create a logger with [ctor@Logger.new], optionally
  * configure it with [method@Logger.set_request_filter],
  * [method@Logger.set_response_filter], and [method@Logger.set_printer], and
  * then attach it to a session (or multiple sessions) with
@@ -49,11 +49,11 @@
  * > Content-Type: text/plain
  * > Connection: close
  *
- * < HTTP/1.1 201 Created
- * < Soup-Debug-Timestamp: 1200171744
- * < Soup-Debug: SoupMessage 1 (0x617000)
- * < Date: Sun, 12 Jan 2008 21:02:24 GMT
- * < Content-Length: 0
+ * &lt; HTTP/1.1 201 Created
+ * &lt; Soup-Debug-Timestamp: 1200171744
+ * &lt; Soup-Debug: SoupMessage 1 (0x617000)
+ * &lt; Date: Sun, 12 Jan 2008 21:02:24 GMT
+ * &lt; Content-Length: 0
  * ```
  *
  * The `Soup-Debug-Timestamp` line gives the time (as a `time_t`) when the
@@ -63,7 +63,7 @@
  * [class@Session], [class@Message], and [class@Gio.Socket] involved; the hex
  * numbers are the addresses of the objects in question (which may be useful if
  * you are running in a debugger). The decimal IDs are simply counters that
- * uniquely identify objects across the lifetime of the [class@Logger]. In
+ * uniquely identify objects across the lifetime of the #SoupLogger. In
  * particular, this can be used to identify when multiple messages are sent
  * across the same connection.
  *
@@ -75,7 +75,7 @@
  * from the network (from the [signal@Message::got-body] or
  * [signal@Message::got-informational] signal), which means that the
  * [signal@Message::got-headers] signal, and anything triggered off it (such as
- * [signal@Message::authenticate]) will be emitted *before* the response headers are
+ * #SoupMessage::authenticate) will be emitted *before* the response headers are
  * actually logged.
  *
  * If the response doesn't happen to trigger the [signal@Message::got-body] nor
@@ -196,14 +196,11 @@ soup_logger_content_processor_wrap_input (SoupContentProcessor *processor,
         SoupLogger *logger = SOUP_LOGGER (processor);
         SoupLoggerPrivate *priv = soup_logger_get_instance_private (logger);
         SoupLoggerInputStream *stream;
-        SoupLoggerLogLevel log_level = SOUP_LOGGER_LOG_NONE;
+        SoupLoggerLogLevel log_level;
 
-        if (priv->request_filter || priv->response_filter) {
-                if (priv->request_filter)
-                        log_level = priv->request_filter (logger, msg, priv->request_filter_data);
-                if (priv->response_filter)
-                        log_level = MAX(log_level, priv->response_filter (logger, msg, priv->response_filter_data));
-        }
+        if (priv->request_filter)
+                log_level = priv->request_filter (logger, msg,
+                                                  priv->response_filter_data);
         else
                 log_level = priv->level;
 
@@ -374,7 +371,7 @@ soup_logger_class_init (SoupLoggerClass *logger_class)
  * soup_logger_new:
  * @level: the debug level
  *
- * Creates a new [class@Logger] with the given debug level.
+ * Creates a new #SoupLogger with the given debug level.
  *
  * If you need finer control over what message parts are and aren't
  * logged, use [method@Logger.set_request_filter] and
@@ -593,15 +590,6 @@ soup_logger_print (SoupLogger *logger, SoupLoggerLogLevel level,
 	g_free (data);
 }
 
-static void
-soup_logger_print_decorative_newline (SoupLogger *logger)
-{
-	SoupLoggerPrivate *priv = soup_logger_get_instance_private (logger);
-	if (!priv->printer) {
-		g_print("\n");
-	}
-}
-
 static void
 soup_logger_print_basic_auth (SoupLogger *logger, const char *value)
 {
@@ -692,8 +680,6 @@ print_request (SoupLogger *logger, SoupMessage *msg,
 	if (log_level == SOUP_LOGGER_LOG_MINIMAL)
 		return;
 
-	soup_logger_print (logger, SOUP_LOGGER_LOG_HEADERS, '>', "Soup-Host: %s", g_uri_get_host (uri));
-
 	soup_message_headers_iter_init (&iter, soup_message_get_request_headers (msg));
 	while (soup_message_headers_iter_next (&iter, &name, &value)) {
 		if (!g_ascii_strcasecmp (name, "Authorization") &&
@@ -706,20 +692,17 @@ print_request (SoupLogger *logger, SoupMessage *msg,
 	}
 
 	if (log_level == SOUP_LOGGER_LOG_HEADERS)
-		goto print_newline_and_return;
+		return;
 
 	/* will be logged in get_informational */
 	if (soup_message_headers_get_expectations (soup_message_get_request_headers (msg)) == SOUP_EXPECTATION_CONTINUE)
-		goto print_newline_and_return;
+		return;
 
 	if (!g_hash_table_steal_extended (priv->request_bodies, msg, NULL, (gpointer *)&body))
-		goto print_newline_and_return;
+		return;
 
 	soup_logger_print (logger, SOUP_LOGGER_LOG_BODY, '>', "\n%s", body->str);
 	g_string_free (body, TRUE);
-
-print_newline_and_return:
-	soup_logger_print_decorative_newline (logger);
 }
 
 static void
@@ -754,7 +737,7 @@ print_response (SoupLogger *logger, SoupMessage *msg)
 			   soup_logger_get_id (logger, msg), msg);
 
 	if (log_level == SOUP_LOGGER_LOG_MINIMAL)
-		goto print_newline_and_return;
+		return;
 
 	soup_message_headers_iter_init (&iter, soup_message_get_response_headers (msg));
 	while (soup_message_headers_iter_next (&iter, &name, &value)) {
@@ -763,16 +746,13 @@ print_response (SoupLogger *logger, SoupMessage *msg)
 	}
 
 	if (log_level == SOUP_LOGGER_LOG_HEADERS)
-		goto print_newline_and_return;
+		return;
 
 	if (!g_hash_table_steal_extended (priv->response_bodies, msg, NULL, (gpointer *)&body))
-		goto print_newline_and_return;
+		return;
 
 	soup_logger_print (logger, SOUP_LOGGER_LOG_BODY, '<', "\n%s", body->str);
 	g_string_free (body, TRUE);
-
-print_newline_and_return:
-	soup_logger_print_decorative_newline (logger);
 }
 
 static void
@@ -789,6 +769,7 @@ finished (SoupMessage *msg, gpointer user_data)
 
         g_mutex_lock (&priv->mutex);
 	print_response (logger, msg);
+	soup_logger_print (logger, SOUP_LOGGER_LOG_MINIMAL, ' ', "\n");
         g_mutex_unlock (&priv->mutex);
 }
 
@@ -810,6 +791,7 @@ got_informational (SoupMessage *msg, gpointer user_data)
 
         g_signal_handlers_disconnect_by_func (msg, finished, logger);
         print_response (logger, msg);
+        soup_logger_print (logger, SOUP_LOGGER_LOG_MINIMAL, ' ', "\n");
 
         if (!g_hash_table_steal_extended (priv->response_bodies, msg, NULL, (gpointer *)&body)) {
                 g_mutex_unlock (&priv->mutex);
@@ -825,7 +807,7 @@ got_informational (SoupMessage *msg, gpointer user_data)
                                            '>', "%s", body->str);
                 }
 
-                soup_logger_print_decorative_newline (logger);
+                soup_logger_print (logger, SOUP_LOGGER_LOG_MINIMAL, ' ', "\n");
         }
 
         g_string_free (body, TRUE);
@@ -844,6 +826,7 @@ got_body (SoupMessage *msg, gpointer user_data)
 	g_signal_handlers_disconnect_by_func (msg, finished, logger);
 
 	print_response (logger, msg);
+	soup_logger_print (logger, SOUP_LOGGER_LOG_MINIMAL, ' ', "\n");
 
         g_mutex_unlock (&priv->mutex);
 }
@@ -879,6 +862,7 @@ wrote_body (SoupMessage *msg, gpointer user_data)
 
         g_mutex_lock (&priv->mutex);
 	print_request (logger, msg, socket, restarted);
+	soup_logger_print (logger, SOUP_LOGGER_LOG_MINIMAL, ' ', "\n");
         g_mutex_unlock (&priv->mutex);
 }
 
