@@ -83,6 +83,7 @@ test_init (int argc, char **argv, GOptionEntry *entries)
 	g_set_prgname (name);
 
 	g_test_init (&argc, &argv, NULL);
+	g_test_set_nonfatal_assertions ();
 	g_test_bug_base ("https://bugzilla.gnome.org/");
 
 	opts = g_option_context_new (NULL);
@@ -142,8 +143,6 @@ debug_printf (int level, const char *format, ...)
 	va_start (args, format);
 	g_vprintf (format, args);
 	va_end (args);
-
-	fflush (stdout);
 }
 
 gboolean
@@ -606,11 +605,9 @@ static gboolean
 add_listener_in_thread (gpointer user_data)
 {
 	AddListenerData *data = user_data;
-	GUri *uri;
 
-	uri = add_listener (data->server, data->scheme, data->host);
+	data->uri = add_listener (data->server, data->scheme, data->host);
 	g_mutex_lock (&data->mutex);
-	data->uri = uri;
 	g_cond_signal (&data->cond);
 	g_mutex_unlock (&data->mutex);
 
@@ -642,9 +639,9 @@ soup_test_server_get_uri (SoupServer    *server,
 		data.host = host;
 		data.uri = NULL;
 
+		g_mutex_lock (&data.mutex);
 		soup_add_completion (context, add_listener_in_thread, &data);
 
-		g_mutex_lock (&data.mutex);
 		while (!data.uri)
 			g_cond_wait (&data.cond, &data.mutex);
 
